@@ -177,6 +177,20 @@ is_hyper (const gchar *string)
 	  (string[6] == '>'));
 }
 
+static inline gboolean
+is_primary (const gchar *string)
+{
+	return ((string[0] == '<') &&
+		(string[1] == 'p' || string[1] == 'P') &&
+		(string[2] == 'r' || string[2] == 'R') &&
+		(string[3] == 'i' || string[3] == 'I') &&
+		(string[4] == 'm' || string[4] == 'M') &&
+		(string[5] == 'a' || string[5] == 'A') &&
+		(string[6] == 'r' || string[6] == 'R') &&
+		(string[7] == 'y' || string[7] == 'Y') &&
+		(string[8] == '>'));
+}
+
 static inline gboolean
 is_keycode (const gchar *string)
 {
@@ -312,6 +326,12 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
 	      len -= 7;
 	      mods |= EGG_VIRTUAL_SUPER_MASK;
 	    }
+	  else if (len >= 9 && is_primary (accelerator))
+	    {
+	      accelerator += 9;
+	      len -= 9;
+	      mods |= EGG_VIRTUAL_CONTROL_MASK;
+	    }
 	  else
 	    {
 	      gchar last_ch;
@@ -327,10 +347,14 @@ egg_accelerator_parse_virtual (const gchar            *accelerator,
 	}
       else
 	{
-          keyval = gdk_keyval_from_name (accelerator);
+	  if (g_str_has_prefix (accelerator, "0x"))
+	    keyval = GDK_KEY_VoidSymbol;
+	  else
+            keyval = gdk_keyval_from_name (accelerator);
 
           if (keyval == GDK_KEY_VoidSymbol)
 	    {
+	      keyval = 0;
 	      /* If keyval is 0, then maybe it's a keycode.  Check for 0x## */
 	      if (len >= 4 && is_keycode (accelerator))
 		{
